@@ -34,6 +34,7 @@
 
 #include "eog-debug.h"
 #include "photos-application.h"
+#include "photos-camera-cache.h"
 #include "photos-dlna-renderers-dialog.h"
 #include "photos-gom-miner.h"
 #include "photos-item-manager.h"
@@ -62,11 +63,12 @@ struct _PhotosApplicationPrivate
   GtkWidget *main_window;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
+  PhotosCameraCache *camera_cache;
   PhotosModeController *mode_cntrlr;
 };
 
 
-G_DEFINE_TYPE (PhotosApplication, photos_application, GTK_TYPE_APPLICATION)
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosApplication, photos_application, GTK_TYPE_APPLICATION)
 
 
 enum
@@ -421,6 +423,12 @@ photos_application_startup (GApplication *application)
 
   priv->item_mngr = photos_item_manager_new ();
   priv->src_mngr = photos_source_manager_new ();
+
+  /* A dummy reference to keep it alive during the lifetime of the
+   * application.
+   */
+  priv->camera_cache = photos_camera_cache_new ();
+
   priv->mode_cntrlr = photos_mode_controller_new ();
 
   action = g_simple_action_new ("about", NULL);
@@ -548,19 +556,17 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->flickr_miner);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
+  g_clear_object (&priv->camera_cache);
   g_clear_object (&priv->mode_cntrlr);
 
-  G_OBJECT_CLASS (photos_application_parent_class)
-    ->dispose (object);
+  G_OBJECT_CLASS (photos_application_parent_class)->dispose (object);
 }
 
 
 static void
 photos_application_init (PhotosApplication *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                            PHOTOS_TYPE_APPLICATION,
-                                            PhotosApplicationPrivate);
+  self->priv = photos_application_get_instance_private (self);
   eog_debug_init ();
 }
 
@@ -577,8 +583,6 @@ photos_application_class_init (PhotosApplicationClass *class)
   application_class->startup = photos_application_startup;
 
   /* TODO: Add miners-changed signal */
-
-  g_type_class_add_private (class, sizeof (PhotosApplicationPrivate));
 }
 
 
