@@ -29,6 +29,7 @@
 
 #include <gio/gio.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include <tracker-sparql.h>
 
 #include "photos-base-item.h"
@@ -36,6 +37,7 @@
 #include "photos-delete-item-job.h"
 #include "photos-print-operation.h"
 #include "photos-query.h"
+#include "photos-selection-controller.h"
 #include "photos-single-item-job.h"
 #include "photos-utils.h"
 
@@ -47,10 +49,12 @@ struct _PhotosBaseItemPrivate
   GeglNode *graph;
   GeglNode *node;
   GeglRectangle bbox;
-  GMutex mutex_create_thumbnail;
   GMutex mutex_download;
   GMutex mutex;
+  GQuark equipment;
+  GQuark flash;
   PhotosCollectionIconWatcher *watcher;
+  PhotosSelectionController *sel_cntrlr;
   TrackerSparqlCursor *cursor;
   gboolean collection;
   gboolean failed_thumbnailing;
@@ -67,8 +71,14 @@ struct _PhotosBaseItemPrivate
   gchar *resource_urn;
   gchar *type_description;
   gchar *uri;
+  gdouble exposure_time;
+  gdouble fnumber;
+  gdouble focal_length;
+  gdouble iso_speed;
   gint64 date_created;
+  gint64 height;
   gint64 mtime;
+  gint64 width;
 };
 
 enum
@@ -88,38 +98,15 @@ enum
 static guint signals[LAST_SIGNAL] = { 0 };
 
 
-G_DEFINE_TYPE (PhotosBaseItem, photos_base_item, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosBaseItem, photos_base_item, G_TYPE_OBJECT);
 
 
-typedef struct _PhotosBaseItemAsyncStrData PhotosBaseItemAsyncStrData;
+G_LOCK_DEFINE (create_thumbnail);
 
-struct _PhotosBaseItemAsyncStrData
-{
-  gchar *str;
-};
 
 static void photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor *cursor);
 
 
-static PhotosBaseItemAsyncStrData *
-photos_base_item_async_str_data_new (gchar *str)
-{
-  PhotosBaseItemAsyncStrData *data;
-
-  data = g_slice_new0 (PhotosBaseItemAsyncStrData);
-  data->str = str;
-  return data;
-}
-
-
-static void
-photos_base_item_async_str_data_free (PhotosBaseItemAsyncStrData *data)
-{
-  g_free (data->str);
-  g_slice_free (PhotosBaseItemAsyncStrData, data);
-}
-
-
 static GIcon *
 photos_base_item_create_symbolic_emblem (const gchar *name)
 {
@@ -232,11 +219,10 @@ photos_base_item_create_thumbnail_in_thread_func (GSimpleAsyncResult *simple,
                                                   GCancellable *cancellable)
 {
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (object);
-  PhotosBaseItemPrivate *priv = self->priv;
   GError *error;
   gboolean op_res;
 
-  g_mutex_lock (&priv->mutex_create_thumbnail);
+  G_LOCK (create_thumbnail);
 
   error = NULL;
   op_res = PHOTOS_BASE_ITEM_GET_CLASS (self)->create_thumbnail (self, cancellable, &error);
@@ -245,7 +231,7 @@ photos_base_item_create_thumbnail_in_thread_func (GSimpleAsyncResult *simple,
 
   g_simple_async_result_set_op_res_gboolean (simple, op_res);
 
-  g_mutex_unlock (&priv->mutex_create_thumbnail);
+  G_UNLOCK (create_thumbnail);
 }
 
 
@@ -317,12 +303,29 @@ photos_base_item_default_open (PhotosBaseItem *self, GdkScreen *screen, guint32
 
 
 static void
-photos_base_item_download_in_thread_func (GSimpleAsyncResult *simple, GObject *object, GCancellable *cancellable)
+photos_base_item_default_update_type_description (PhotosBaseItem *self)
 {
-  PhotosBaseItem *self = PHOTOS_BASE_ITEM (object);
+  PhotosBaseItemPrivate *priv = self->priv;
+  gchar *description = NULL;
+
+  if (priv->collection)
+    description = g_strdup (_("Album"));
+  else if (priv->mime_type != NULL)
+    description = g_content_type_get_description (priv->mime_type);
+
+  priv->type_description = description;
+}
+
+
+static void
+photos_base_item_download_in_thread_func (GTask *task,
+                                          gpointer source_object,
+                                          gpointer task_data,
+                                          GCancellable *cancellable)
+{
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   PhotosBaseItemPrivate *priv = self->priv;
   GError *error;
-  PhotosBaseItemAsyncStrData *op_res;
   gchar *path;
 
   g_mutex_lock (&priv->mutex_download);
@@ -330,11 +333,14 @@ photos_base_item_download_in_thread_func (GSimpleAsyncResult *simple, GObject *o
   error = NULL;
   path = photos_base_item_download (self, cancellable, &error);
   if (error != NULL)
-    g_simple_async_result_take_error (simple, error);
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
 
-  op_res = photos_base_item_async_str_data_new (path);
-  g_simple_async_result_set_op_res_gpointer (simple, op_res, (GDestroyNotify) photos_base_item_async_str_data_free);
+  g_task_return_pointer (task, path, g_free);
 
+ out:
   g_mutex_unlock (&priv->mutex_download);
 }
 
@@ -594,9 +600,12 @@ photos_base_item_load_in_thread_func (GSimpleAsyncResult *simple, GObject *objec
 
   node = photos_base_item_load (self, cancellable, &error);
   if (error != NULL)
-    g_simple_async_result_take_error (simple, error);
-
-  g_simple_async_result_set_op_res_gpointer (simple, (gpointer) node, g_object_unref);
+    {
+      g_simple_async_result_take_error (simple, error);
+      g_simple_async_result_set_op_res_gpointer (simple, NULL, NULL);
+    }
+  else
+    g_simple_async_result_set_op_res_gpointer (simple, (gpointer) node, g_object_unref);
 
   g_mutex_unlock (&priv->mutex);
 }
@@ -686,6 +695,8 @@ photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor
   PhotosBaseItemPrivate *priv = self->priv;
   GTimeVal timeval;
   const gchar *date_created;
+  const gchar *equipment;
+  const gchar *flash;
   const gchar *mtime;
   const gchar *title;
   const gchar *uri;
@@ -726,10 +737,32 @@ photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor
     title = "";
   priv->name = g_strdup (title);
 
+  priv->width = tracker_sparql_cursor_get_integer (cursor, PHOTOS_QUERY_COLUMNS_WIDTH);
+  priv->height = tracker_sparql_cursor_get_integer (cursor, PHOTOS_QUERY_COLUMNS_HEIGHT);
+
+  equipment = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_EQUIPMENT, NULL);
+  priv->equipment = g_quark_from_string (equipment);
+
+  priv->exposure_time = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_EXPOSURE_TIME);
+  priv->fnumber = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_FNUMBER);
+  priv->focal_length = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_FOCAL_LENGTH);
+  priv->iso_speed = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_ISO_SPEED);
+
+  flash = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_FLASH, NULL);
+  priv->flash = g_quark_from_string (flash);
+
   photos_base_item_refresh_icon (self);
 }
 
 
+static void
+photos_base_item_print_operation_done (PhotosBaseItem *self, GtkPrintOperationResult result)
+{
+  if (result == GTK_PRINT_OPERATION_RESULT_APPLY)
+      photos_selection_controller_set_selection_mode (self->priv->sel_cntrlr, FALSE);
+}
+
+
 static void
 photos_base_item_print_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -743,6 +776,12 @@ photos_base_item_print_load (GObject *source_object, GAsyncResult *res, gpointer
     goto out;
 
   print_op = photos_print_operation_new (self, node);
+  g_signal_connect_data (print_op,
+                         "done",
+                         G_CALLBACK (photos_base_item_print_operation_done),
+                         g_object_ref (self),
+                         (GClosureNotify) g_object_unref,
+                         G_CONNECT_SWAPPED);
   gtk_print_operation_run (print_op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, toplevel, NULL);
 
  out:
@@ -751,18 +790,6 @@ photos_base_item_print_load (GObject *source_object, GAsyncResult *res, gpointer
 }
 
 
-static void
-photos_base_item_update_type_description (PhotosBaseItem *self)
-{
-  PhotosBaseItemPrivate *priv = self->priv;
-
-  if (priv->mime_type == NULL)
-    return;
-
-  priv->type_description = g_content_type_get_description (priv->mime_type);
-}
-
-
 static void
 photos_base_item_constructed (GObject *object)
 {
@@ -786,6 +813,7 @@ photos_base_item_dispose (GObject *object)
   g_clear_object (&priv->icon);
   g_clear_object (&priv->pristine_icon);
   g_clear_object (&priv->watcher);
+  g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->cursor);
 
   G_OBJECT_CLASS (photos_base_item_parent_class)->dispose (object);
@@ -809,7 +837,6 @@ photos_base_item_finalize (GObject *object)
   g_free (priv->type_description);
   g_free (priv->uri);
 
-  g_mutex_clear (&priv->mutex_create_thumbnail);
   g_mutex_clear (&priv->mutex_download);
   g_mutex_clear (&priv->mutex);
 
@@ -863,12 +890,13 @@ photos_base_item_init (PhotosBaseItem *self)
 {
   PhotosBaseItemPrivate *priv;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_BASE_ITEM, PhotosBaseItemPrivate);
+  self->priv = photos_base_item_get_instance_private (self);
   priv = self->priv;
 
-  g_mutex_init (&priv->mutex_create_thumbnail);
   g_mutex_init (&priv->mutex_download);
   g_mutex_init (&priv->mutex);
+
+  priv->sel_cntrlr = photos_selection_controller_new ();
 }
 
 
@@ -884,7 +912,7 @@ photos_base_item_class_init (PhotosBaseItemClass *class)
   object_class->set_property = photos_base_item_set_property;
   class->open = photos_base_item_default_open;
   class->set_favorite = photos_base_item_default_set_favorite;
-  class->update_type_description = photos_base_item_update_type_description;
+  class->update_type_description = photos_base_item_default_update_type_description;
 
   g_object_class_install_property (object_class,
                                    PROP_CURSOR,
@@ -920,8 +948,6 @@ photos_base_item_class_init (PhotosBaseItemClass *class)
                                         g_cclosure_marshal_VOID__VOID,
                                         G_TYPE_NONE,
                                         0);
-
-  g_type_class_add_private (class, sizeof (PhotosBaseItemPrivate));
 }
 
 
@@ -953,44 +979,27 @@ photos_base_item_download_async (PhotosBaseItem *self,
                                  GAsyncReadyCallback callback,
                                  gpointer user_data)
 {
-  GSimpleAsyncResult *simple;
+  GTask *task;
 
-  simple = g_simple_async_result_new (G_OBJECT (self),
-                                      callback,
-                                      user_data,
-                                      photos_base_item_download_async);
-  g_simple_async_result_set_check_cancellable (simple, cancellable);
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_check_cancellable (task, TRUE);
+  g_task_set_source_tag (task, photos_base_item_download_async);
 
-  g_simple_async_result_run_in_thread (simple,
-                                       photos_base_item_download_in_thread_func,
-                                       G_PRIORITY_DEFAULT,
-                                       cancellable);
-  g_object_unref (simple);
+  g_task_run_in_thread (task, photos_base_item_download_in_thread_func);
+  g_object_unref (task);
 }
 
 
 gchar *
 photos_base_item_download_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
 {
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
-  PhotosBaseItemAsyncStrData *op_res;
-  gchar *ret_val = NULL;
+  GTask *task = G_TASK (res);
 
-  g_return_val_if_fail (g_simple_async_result_is_valid (res,
-                                                        G_OBJECT (self),
-                                                        photos_base_item_download_async),
-                        NULL);
+  g_return_val_if_fail (g_task_is_valid (res, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_download_async, NULL);
   g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
-  if (g_simple_async_result_propagate_error (simple, error))
-    goto out;
-
-  op_res = g_simple_async_result_get_op_res_gpointer (simple);
-  ret_val = op_res->str;
-  op_res->str = NULL;
-
- out:
-  return ret_val;
+  return g_task_propagate_pointer (task, error);
 }
 
 
@@ -1022,6 +1031,48 @@ photos_base_item_get_default_app_name (PhotosBaseItem *self)
 }
 
 
+GQuark
+photos_base_item_get_equipment (PhotosBaseItem *self)
+{
+  return self->priv->equipment;
+}
+
+
+gdouble
+photos_base_item_get_exposure_time (PhotosBaseItem *self)
+{
+  return self->priv->exposure_time;
+}
+
+
+GQuark
+photos_base_item_get_flash (PhotosBaseItem *self)
+{
+  return self->priv->flash;
+}
+
+
+gdouble
+photos_base_item_get_fnumber (PhotosBaseItem *self)
+{
+  return self->priv->fnumber;
+}
+
+
+gdouble
+photos_base_item_get_focal_length (PhotosBaseItem *self)
+{
+  return self->priv->focal_length;
+}
+
+
+gint64
+photos_base_item_get_height (PhotosBaseItem *self)
+{
+  return self->priv->height;
+}
+
+
 GdkPixbuf *
 photos_base_item_get_icon (PhotosBaseItem *self)
 {
@@ -1043,6 +1094,13 @@ photos_base_item_get_identifier (PhotosBaseItem *self)
 }
 
 
+gdouble
+photos_base_item_get_iso_speed (PhotosBaseItem *self)
+{
+  return self->priv->iso_speed;
+}
+
+
 const gchar *
 photos_base_item_get_mime_type (PhotosBaseItem *self)
 {
@@ -1114,6 +1172,13 @@ photos_base_item_get_where (PhotosBaseItem *self)
 }
 
 
+gint64
+photos_base_item_get_width (PhotosBaseItem *self)
+{
+  return self->priv->width;
+}
+
+
 gboolean
 photos_base_item_is_collection (PhotosBaseItem *self)
 {
