@@ -21,18 +21,10 @@
 
 #include "config.h"
 
+#include "photos-dleyna-renderer-device.h"
+#include "photos-dleyna-renderer-push-host.h"
 #include "photos-dlna-renderer.h"
-
-
-typedef enum
-{
-  INIT_NOTHING   = 0,
-  INIT_DEVICE    = 1 << 0,
-  INIT_PUSH_HOST = 1 << 1,
-  INIT_PLAYER    = 1 << 2,
-  INIT_READY     = INIT_DEVICE | INIT_PUSH_HOST | INIT_PLAYER
-} InitStatus;
-
+#include "photos-mpris-player.h"
 
 struct _PhotosDlnaRendererPrivate
 {
@@ -45,8 +37,6 @@ struct _PhotosDlnaRendererPrivate
   DleynaPushHost *push_host;
   MprisPlayer *player;
 
-  InitStatus init_status;
-
   GHashTable *urls_to_item;
 };
 
@@ -66,8 +56,19 @@ static void photos_dlna_renderer_async_initable_iface_init (GAsyncInitableIface
 
 
 G_DEFINE_TYPE_WITH_CODE (PhotosDlnaRenderer, photos_dlna_renderer, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (PhotosDlnaRenderer)
                          G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_INITABLE, photos_dlna_renderer_async_initable_iface_init));
 
+#define RETURN_ON_ERROR(task, error, msg) \
+  do { \
+    if (error != NULL) \
+      { \
+        g_debug ("%s: %s: %s", __func__, msg, error->message); \
+        g_task_return_error (task, error); \
+        g_object_unref (task); \
+        return; \
+      } \
+  } while (0)
 
 static void
 photos_dlna_renderer_dispose (GObject *object)
@@ -157,8 +158,7 @@ photos_dlna_renderer_init (PhotosDlnaRenderer *self)
 {
   PhotosDlnaRendererPrivate *priv;
 
-  self->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_DLNA_RENDERER,
-                                                   PhotosDlnaRendererPrivate);
+  self->priv = priv = photos_dlna_renderer_get_instance_private (self);
 
   priv->urls_to_item = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 }
@@ -236,8 +236,6 @@ photos_dlna_renderer_class_init (PhotosDlnaRendererClass *class)
                                                       G_PARAM_STATIC_NAME |
                                                       G_PARAM_STATIC_BLURB |
                                                       G_PARAM_STATIC_NICK));
-
-  g_type_class_add_private (class, sizeof (PhotosDlnaRendererPrivate));
 }
 
 
@@ -257,38 +255,6 @@ photos_dlna_renderer_new_for_bus (GBusType             bus_type,
 }
 
 
-/* photos_dlna_renderer_init_check_complete:
- *
- * Checks that all the async subtasks have completed and return the result.
- *
- * Note that if the caller does not take ownership of the object (eg. by not
- * calling g_task_propagate_pointer() or in case of errors) this may lead to
- * the object being disposed.
- */
-static gboolean
-photos_dlna_renderer_init_check_complete (PhotosDlnaRenderer *self,
-                                          GTask              *init_task)
-{
-  GError *error;
-
-  g_return_val_if_fail (g_task_is_valid (init_task, self), TRUE);
-
-  if (self->priv->init_status != INIT_READY)
-    return FALSE;
-
-  error = g_task_get_task_data (init_task);
-
-  if (error != NULL)
-    g_task_return_error (init_task, error);
-  else
-    g_task_return_boolean (init_task, TRUE);
-
-  g_object_unref (init_task);
-
-  return TRUE;
-}
-
-
 static void
 photos_dlna_renderer_device_proxy_new_cb (GObject      *source_object,
                                           GAsyncResult *res,
@@ -296,74 +262,15 @@ photos_dlna_renderer_device_proxy_new_cb (GObject      *source_object,
 {
   GTask *init_task = G_TASK (user_data);
   PhotosDlnaRenderer *self;
-  PhotosDlnaRendererPrivate *priv;
   GError *error = NULL;
 
   self = PHOTOS_DLNA_RENDERER (g_task_get_source_object (init_task));
-  priv = self->priv;
 
-  priv->init_status |= INIT_DEVICE;
-  priv->device = dleyna_renderer_device_proxy_new_for_bus_finish (res, &error);
+  self->priv->device = dleyna_renderer_device_proxy_new_for_bus_finish (res, &error);
+  RETURN_ON_ERROR (init_task, error, "Unable to load the RendererDevice interface");
 
-  if (error != NULL)
-    {
-      g_warning ("Unable to load the RendererDevice interface: %s", error->message);
-      g_task_set_task_data (init_task, error, (GDestroyNotify) g_error_free);
-    }
-
-  photos_dlna_renderer_init_check_complete (self, init_task);
-}
-
-
-static void
-photos_dlna_renderer_push_host_proxy_new_cb (GObject      *source_object,
-                                             GAsyncResult *res,
-                                             gpointer      user_data)
-{
-  GTask *init_task = G_TASK (user_data);
-  PhotosDlnaRenderer *self;
-  PhotosDlnaRendererPrivate *priv;
-  GError *error = NULL;
-
-  self = PHOTOS_DLNA_RENDERER (g_task_get_source_object (init_task));
-  priv = self->priv;
-
-  priv->init_status |= INIT_PUSH_HOST;
-  priv->push_host = dleyna_push_host_proxy_new_for_bus_finish (res, &error);
-
-  if (error != NULL)
-    {
-      g_warning ("Unable to load the PushHost interface: %s", error->message);
-      g_task_set_task_data (init_task, error, (GDestroyNotify) g_error_free);
-    }
-
-  photos_dlna_renderer_init_check_complete (self, init_task);
-}
-
-
-static void
-photos_dlna_renderer_player_proxy_new_cb (GObject      *source_object,
-                                          GAsyncResult *res,
-                                          gpointer      user_data)
-{
-  GTask *init_task = G_TASK (user_data);
-  PhotosDlnaRenderer *self;
-  PhotosDlnaRendererPrivate *priv;
-  GError *error = NULL;
-
-  self = PHOTOS_DLNA_RENDERER (g_task_get_source_object (init_task));
-  priv = self->priv;
-
-  priv->init_status |= INIT_PLAYER;
-  priv->player = mpris_player_proxy_new_for_bus_finish (res, &error);
-
-  if (error != NULL)
-    {
-      g_warning ("Unable to load the Player interface: %s", error->message);
-      g_task_set_task_data (init_task, error, (GDestroyNotify) g_error_free);
-    }
-
-  photos_dlna_renderer_init_check_complete (self, init_task);
+  g_task_return_boolean (init_task, TRUE);
+  g_object_unref (init_task);
 }
 
 
@@ -377,24 +284,36 @@ photos_dlna_renderer_init_async (GAsyncInitable      *initable,
   PhotosDlnaRenderer *self = PHOTOS_DLNA_RENDERER (initable);
   PhotosDlnaRendererPrivate *priv = self->priv;
   GTask *init_task;
+  GError *error = NULL;
 
   init_task = g_task_new (initable, cancellable, callback, user_data);
   g_task_set_priority (init_task, io_priority);
 
-  /* Load all three DBus interface proxies asynchronously and then complete
-   * the initialization when all of them are ready */
-  dleyna_renderer_device_proxy_new_for_bus (priv->bus_type, priv->flags, priv->well_known_name,
-                                            priv->object_path, cancellable,
+  priv->push_host = dleyna_push_host_proxy_new_for_bus_sync (priv->bus_type,
+                                                             G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES
+                                                             | G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                                             priv->well_known_name,
+                                                             priv->object_path,
+                                                             NULL,
+                                                             &error);
+  RETURN_ON_ERROR (init_task, error, "Unable to load the PushHost interface");
+
+  priv->player = mpris_player_proxy_new_for_bus_sync (priv->bus_type,
+                                                      G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES
+                                                      | G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                                      priv->well_known_name,
+                                                      priv->object_path,
+                                                      NULL,
+                                                      &error);
+  RETURN_ON_ERROR (init_task, error, "Unable to load the Player interface");
+
+  dleyna_renderer_device_proxy_new_for_bus (priv->bus_type,
+                                            G_DBUS_PROXY_FLAGS_NONE,
+                                            priv->well_known_name,
+                                            priv->object_path,
+                                            cancellable,
                                             photos_dlna_renderer_device_proxy_new_cb,
                                             init_task);
-  dleyna_push_host_proxy_new_for_bus (priv->bus_type, priv->flags, priv->well_known_name,
-                                      priv->object_path, cancellable,
-                                      photos_dlna_renderer_push_host_proxy_new_cb,
-                                      init_task);
-  mpris_player_proxy_new_for_bus (priv->bus_type, priv->flags, priv->well_known_name,
-                                  priv->object_path, cancellable,
-                                  photos_dlna_renderer_player_proxy_new_cb,
-                                  init_task);
 }
 
 
@@ -430,18 +349,7 @@ photos_dlna_renderer_async_initable_iface_init (GAsyncInitableIface *iface)
 const gchar *
 photos_dlna_renderer_get_object_path (PhotosDlnaRenderer *self)
 {
-  PhotosDlnaRendererPrivate *priv = self->priv;
-
-  return priv->object_path;
-}
-
-
-DleynaRendererDevice *
-photos_dlna_renderer_get_device (PhotosDlnaRenderer *self)
-{
-  PhotosDlnaRendererPrivate *priv = self->priv;
-
-  return priv->device;
+  return self->priv->object_path;
 }
 
 
@@ -455,17 +363,10 @@ photos_dlna_renderer_share_play_cb (GObject      *source_object,
   GError *error = NULL;
 
   mpris_player_call_play_finish (MPRIS_PLAYER (source_object), res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Failed to call the Play method: %s", error->message);
-      g_task_return_error (task, error);
-      goto out;
-    }
+  RETURN_ON_ERROR (task, error, "Failed to call the Play method");
 
   item = g_object_get_data (G_OBJECT (task), "item");
   g_task_return_pointer (task, g_object_ref (item), g_object_unref);
-
-out:
   g_object_unref (task);
 }
 
@@ -484,13 +385,7 @@ photos_dlna_renderer_share_open_uri_cb (GObject      *source_object,
   priv = self->priv;
 
   mpris_player_call_open_uri_finish (MPRIS_PLAYER (source_object), res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Failed to call the OpenUri method: %s", error->message);
-      g_task_return_error (task, error);
-      g_object_unref (task);
-      return;
-    }
+  RETURN_ON_ERROR (task, error, "Failed to call the OpenUri method");
 
   /* 3) Mpris.Player.Play() */
   mpris_player_call_play (priv->player,
@@ -516,13 +411,7 @@ photos_dlna_renderer_share_host_file_cb (GObject      *source_object,
   priv = self->priv;
 
   dleyna_push_host_call_host_file_finish (DLEYNA_PUSH_HOST (source_object), &hosted_url, res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Failed to call the HostFile method: %s", error->message);
-      g_task_return_error (task, error);
-      g_object_unref (task);
-      return;
-    }
+  RETURN_ON_ERROR (task, error, "Failed to call the HostFile method");
 
   item = g_object_get_data (G_OBJECT (task), "item");
   g_hash_table_replace (priv->urls_to_item, hosted_url, g_object_ref (item));
@@ -551,13 +440,7 @@ photos_dlna_renderer_share_download_cb (GObject *source_object,
 
   error = NULL;
   filename = photos_base_item_download_finish (PHOTOS_BASE_ITEM (source_object), res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to extract the local filename for the shared item: %s", error->message);
-      g_task_return_error (task, error);
-      g_object_unref (task);
-      return;
-    }
+  RETURN_ON_ERROR (task, error, "Unable to extract the local filename for the shared item");
 
   /* This will call a sequence of DBus methods to send the item to the DMR:
    * 1) DleynaRenderer.PushHost.HostFile (filename)
@@ -600,7 +483,7 @@ photos_dlna_renderer_share_finish (PhotosDlnaRenderer *self,
                                    GAsyncResult       *res,
                                    GError            **error)
 {
-  g_return_val_if_fail (g_task_is_valid (res, G_OBJECT (self)), FALSE);
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
 
   return g_task_propagate_pointer (G_TASK (res), error);
 }
@@ -665,13 +548,7 @@ photos_dlna_renderer_unshare_download_cb (GObject *source_object,
 
   error = NULL;
   filename = photos_base_item_download_finish (PHOTOS_BASE_ITEM (source_object), res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to extract the local filename for the shared item: %s", error->message);
-      g_task_return_error (task, error);
-      g_object_unref (task);
-      return;
-    }
+  RETURN_ON_ERROR (task, error, "Unable to extract the local filename for the shared item");
 
   dleyna_push_host_call_remove_file (self->priv->push_host,
                                      filename,
@@ -703,7 +580,7 @@ photos_dlna_renderer_unshare_finish (PhotosDlnaRenderer *self,
                                      GAsyncResult       *res,
                                      GError            **error)
 {
-  g_return_if_fail (g_task_is_valid (res, G_OBJECT (self)));
+  g_return_if_fail (g_task_is_valid (res, self));
 
   g_task_propagate_boolean (G_TASK (res), error);
 }
@@ -767,7 +644,7 @@ photos_dlna_renderer_unshare_all_finish (PhotosDlnaRenderer *self,
                                          GAsyncResult       *res,
                                          GError            **error)
 {
-  g_return_if_fail (g_task_is_valid (res, G_OBJECT (self)));
+  g_return_if_fail (g_task_is_valid (res, self));
 
   g_task_propagate_boolean (G_TASK (res), error);
 }
@@ -779,48 +656,36 @@ photos_dlna_renderer_device_get_icon_cb (GObject      *source_object,
                                          gpointer      user_data)
 {
   GTask *task = G_TASK (user_data);
-  GInputStream *icon_stream = NULL;
-  GdkPixbuf *pixbuf = NULL;
-  GVariant *icon_variant = NULL;
-  GBytes *icon_bytes = NULL;
+  GInputStream *icon_stream;
+  GdkPixbuf *pixbuf;
+  GVariant *icon_variant;
+  GBytes *icon_bytes;
   const gchar *icon_data;
   gsize icon_data_size;
-  gchar *mimetype = NULL;
   GError *error = NULL;
 
   /* The icon data is forced to be a GVariant since the GDBus bindings assume
    * bytestrings (type 'ay') to be nul-terminated and thus do not return the length
    * of the buffer */
   dleyna_renderer_device_call_get_icon_finish (DLEYNA_RENDERER_DEVICE (source_object),
-                                               &icon_variant, &mimetype, res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Failed to call the GetIcon method: %s", error->message);
-      g_task_return_error (task, error);
-      goto out;
-    }
+                                               &icon_variant, NULL, res, &error);
+  RETURN_ON_ERROR (task, error, "Failed to call the GetIcon method");
 
   /* We know that the serialization of variant containing just a byte array
    * 'ay' is the byte  array itself */
   icon_bytes = g_variant_get_data_as_bytes (icon_variant);
+  g_variant_unref (icon_variant);
+
   icon_data = g_bytes_get_data (icon_bytes, &icon_data_size);
+  g_bytes_unref (icon_bytes);
+
   icon_stream = g_memory_input_stream_new_from_data (icon_data, icon_data_size, NULL);
-
   pixbuf = gdk_pixbuf_new_from_stream (icon_stream, g_task_get_cancellable (task), &error);
-  if (error != NULL)
-    {
-      g_warning ("Failed to parse icon data: %s", error->message);
-      g_task_return_error (task, error);
-      goto out;
-    }
+  g_object_unref (icon_stream);
 
+  RETURN_ON_ERROR (task, error, "Failed to parse icon data");
   g_task_return_pointer (task, pixbuf, g_object_unref);
 
-out:
-  g_free (mimetype);
-  g_clear_pointer (&icon_variant, g_variant_unref);
-  g_clear_pointer (&icon_bytes, g_bytes_unref);
-  g_clear_object (&icon_stream);
   g_object_unref (task);
 }
 
@@ -835,6 +700,15 @@ photos_dlna_renderer_get_friendly_name (PhotosDlnaRenderer *self)
 }
 
 
+const gchar *
+photos_dlna_renderer_get_udn (PhotosDlnaRenderer *self)
+{
+  PhotosDlnaRendererPrivate *priv = self->priv;
+
+  return dleyna_renderer_device_get_udn (priv->device);
+}
+
+
 void
 photos_dlna_renderer_get_icon (PhotosDlnaRenderer  *self,
                                const gchar         *resolution,
@@ -859,7 +733,7 @@ photos_dlna_renderer_get_icon_finish (PhotosDlnaRenderer *self,
                                       GAsyncResult       *res,
                                       GError            **error)
 {
-  g_return_if_fail (g_task_is_valid (res, G_OBJECT (self)));
+  g_return_if_fail (g_task_is_valid (res, self));
 
   return GDK_PIXBUF (g_task_propagate_pointer (G_TASK (res), error));
 }
