@@ -45,7 +45,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObject *photos_dlna_renderers_manager_singleton = NULL;
 
-G_DEFINE_TYPE (PhotosDlnaRenderersManager, photos_dlna_renderers_manager, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosDlnaRenderersManager, photos_dlna_renderers_manager, G_TYPE_OBJECT);
 
 static void
 photos_dlna_renderers_manager_dispose (GObject *object)
@@ -68,7 +68,6 @@ photos_dlna_renderers_manager_renderer_new_cb (GObject      *source_object,
   PhotosDlnaRenderersManager *self = PHOTOS_DLNA_RENDERERS_MANAGER (user_data);
   PhotosDlnaRenderersManagerPrivate *priv = self->priv;
   PhotosDlnaRenderer *renderer;
-  DleynaRendererDevice *device;
   const gchar *object_path;
   GError *error = NULL;
 
@@ -80,11 +79,10 @@ photos_dlna_renderers_manager_renderer_new_cb (GObject      *source_object,
       return;
     }
 
-  device = photos_dlna_renderer_get_device (renderer);
   object_path = photos_dlna_renderer_get_object_path (renderer);
   g_debug ("%s '%s' %s %s", __func__,
-           dleyna_renderer_device_get_friendly_name (device),
-           dleyna_renderer_device_get_udn (device),
+           photos_dlna_renderer_get_friendly_name (renderer),
+           photos_dlna_renderer_get_udn (renderer),
            object_path);
   g_hash_table_insert (priv->renderers, (gpointer) object_path, renderer);
   g_signal_emit (self, signals[RENDERER_FOUND], 0, renderer);
@@ -111,16 +109,14 @@ photos_dlna_renderers_manager_renderer_lost_cb (PhotosDlnaRenderersManager *self
 {
   PhotosDlnaRenderersManagerPrivate *priv = self->priv;
   PhotosDlnaRenderer *renderer;
-  DleynaRendererDevice *device;
 
   renderer = PHOTOS_DLNA_RENDERER (g_hash_table_lookup (priv->renderers, object_path));
   g_return_if_fail (renderer != NULL);
 
   g_hash_table_steal (priv->renderers, object_path);
-  device = photos_dlna_renderer_get_device (renderer);
   g_debug ("%s '%s' %s %s", __func__,
-           dleyna_renderer_device_get_friendly_name (device),
-           dleyna_renderer_device_get_udn (device),
+           photos_dlna_renderer_get_friendly_name (renderer),
+           photos_dlna_renderer_get_udn (renderer),
            object_path);
   g_signal_emit (self, signals[RENDERER_LOST], 0, renderer);
   g_object_unref (renderer);
@@ -200,8 +196,7 @@ photos_dlna_renderers_manager_init (PhotosDlnaRenderersManager *self)
 {
   PhotosDlnaRenderersManagerPrivate *priv;
 
-  self->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_DLNA_RENDERERS_MANAGER,
-                                                   PhotosDlnaRenderersManagerPrivate);
+  self->priv = priv = photos_dlna_renderers_manager_get_instance_private (self);
 
   dleyna_manager_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                                     G_DBUS_PROXY_FLAGS_NONE,
@@ -230,8 +225,6 @@ photos_dlna_renderers_manager_class_init (PhotosDlnaRenderersManagerClass *class
                                           G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                           g_cclosure_marshal_VOID__OBJECT,
                                           G_TYPE_NONE, 1, PHOTOS_TYPE_DLNA_RENDERER);
-
-  g_type_class_add_private (class, sizeof (PhotosDlnaRenderersManagerPrivate));
 }
 
 PhotosDlnaRenderersManager *
