@@ -29,6 +29,7 @@
 #include <gegl.h>
 #include <glib/gi18n.h>
 
+#include "photos-application.h"
 #include "photos-embed.h"
 #include "photos-empty-results-box.h"
 #include "photos-error-box.h"
@@ -71,7 +72,7 @@ struct _PhotosEmbedPrivate
 };
 
 
-G_DEFINE_TYPE (PhotosEmbed, photos_embed, GTK_TYPE_BOX);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosEmbed, photos_embed, GTK_TYPE_BOX);
 
 
 static void photos_embed_changes_pending (PhotosEmbed *self, GHashTable *changes);
@@ -90,13 +91,31 @@ photos_embed_clear_load_timer (PhotosEmbed *self)
 }
 
 
+static void
+photos_embed_prepare_for_preview (PhotosEmbed *self)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  /* TODO: SearchController,
+   *       ErrorHandler
+   */
+
+  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
+  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "preview");
+}
+
+
 static void
 photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
   PhotosEmbedPrivate *priv = self->priv;
   GeglNode *node;
+  GtkListStore *model;
+  GtkTreePath *current_path;
+  GtkWidget *view_container;
   PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+  PhotosWindowMode mode;
 
   photos_embed_clear_load_timer (self);
 
@@ -105,11 +124,52 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
   if (node == NULL)
     goto out;
 
+  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  switch (mode)
+    {
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+      view_container = priv->collections;
+      break;
+
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+      view_container = priv->favorites;
+      break;
+
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+      view_container = priv->overview;
+      break;
+
+    case PHOTOS_WINDOW_MODE_PREVIEW:
+      view_container = NULL;
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+
+  /* If we are already in the preview and navigating using the
+   * buttons, then we don't need this.
+   */
+  if (mode != PHOTOS_WINDOW_MODE_PREVIEW)
+    {
+      current_path = photos_view_container_get_current_path (PHOTOS_VIEW_CONTAINER (view_container));
+      model = photos_view_container_get_model (PHOTOS_VIEW_CONTAINER (view_container));
+      photos_preview_view_set_model (PHOTOS_PREVIEW_VIEW (priv->preview), GTK_TREE_MODEL (model), current_path);
+    }
+
   photos_preview_view_set_node (PHOTOS_PREVIEW_VIEW (priv->preview), node);
 
   /* TODO: set toolbar model */
 
-  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_PREVIEW);
+  /* If we are already in the preview and navigating using the
+   * buttons, then the window-mode-changed signal won't be fired. So
+   * we need to prepare it ourselves.
+   */
+  if (mode != PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_PREVIEW);
+  else
+    photos_embed_prepare_for_preview (self);
+
   photos_mode_controller_set_can_fullscreen (priv->mode_cntrlr, TRUE);
 
  out:
@@ -308,20 +368,6 @@ photos_embed_prepare_for_overview (PhotosEmbed *self)
 }
 
 
-static void
-photos_embed_prepare_for_preview (PhotosEmbed *self)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-
-  /* TODO: SearchController,
-   *       ErrorHandler
-   */
-
-  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
-  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "preview");
-}
-
-
 static void
 photos_embed_set_error (PhotosEmbed *self, const gchar *primary, const gchar *secondary)
 {
@@ -405,18 +451,34 @@ photos_embed_dispose (GObject *object)
   G_OBJECT_CLASS (photos_embed_parent_class)->dispose (object);
 }
 
+
+static void
+photos_embed_window_added (PhotosEmbed *self, GtkWindow *window)
+{
+  gtk_window_set_titlebar (window, self->priv->toolbar);
+}
+
+
 static void
 photos_embed_init (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv;
+  GtkApplication *app;
   gboolean querying;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_EMBED, PhotosEmbedPrivate);
+  self->priv = photos_embed_get_instance_private (self);
   priv = self->priv;
 
+  app = photos_application_new ();
+  g_signal_connect_swapped (app, "window-added", G_CALLBACK (photos_embed_window_added), self);
+  g_object_unref (app);
+
   gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
   gtk_widget_show (GTK_WIDGET (self));
 
+  priv->selection_toolbar = photos_selection_toolbar_new ();
+  gtk_box_pack_end (GTK_BOX (self), priv->selection_toolbar, FALSE, FALSE, 0);
+
   priv->stack_overlay = gtk_overlay_new ();
   gtk_widget_show (priv->stack_overlay);
   gtk_box_pack_end (GTK_BOX (self), priv->stack_overlay, TRUE, TRUE, 0);
@@ -429,7 +491,6 @@ photos_embed_init (PhotosEmbed *self)
 
   priv->toolbar = photos_main_toolbar_new ();
   photos_main_toolbar_set_stack (PHOTOS_MAIN_TOOLBAR (priv->toolbar), GTK_STACK (priv->stack));
-  gtk_box_pack_start (GTK_BOX (self), priv->toolbar, FALSE, FALSE, 0);
 
   priv->ntfctn_mngr = g_object_ref_sink (photos_notification_manager_new ());
   gtk_overlay_add_overlay (GTK_OVERLAY (priv->stack_overlay), priv->ntfctn_mngr);
@@ -445,7 +506,7 @@ photos_embed_init (PhotosEmbed *self)
   priv->favorites = photos_view_container_new (PHOTOS_WINDOW_MODE_FAVORITES);
   gtk_stack_add_titled (GTK_STACK (priv->stack), priv->favorites, "favorites", _("Favorites"));
 
-  priv->preview = photos_preview_view_new ();
+  priv->preview = photos_preview_view_new (GTK_OVERLAY (priv->stack_overlay));
   gtk_stack_add_named (GTK_STACK (priv->stack), priv->preview, "preview");
 
   priv->spinner_box = photos_spinner_box_new ();
@@ -460,9 +521,6 @@ photos_embed_init (PhotosEmbed *self)
   /* TODO: SearchBar.Dropdown, …
    */
 
-  priv->selection_toolbar = photos_selection_toolbar_new ();
-  gtk_overlay_add_overlay (GTK_OVERLAY (priv->stack_overlay), priv->selection_toolbar);
-
   g_signal_connect_object (priv->stack, "notify::visible-child",
                            G_CALLBACK (photos_embed_notify_visible_child),
                            self, G_CONNECT_SWAPPED);
@@ -505,8 +563,6 @@ photos_embed_class_init (PhotosEmbedClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
   object_class->dispose = photos_embed_dispose;
-
-  g_type_class_add_private (class, sizeof (PhotosEmbedPrivate));
 }
 
 
