@@ -37,6 +37,7 @@
 #include "photos-flickr-item.h"
 #include "photos-source.h"
 #include "photos-source-manager.h"
+#include "photos-utils.h"
 
 
 struct _PhotosFlickrItemPrivate
@@ -45,7 +46,7 @@ struct _PhotosFlickrItemPrivate
 };
 
 
-G_DEFINE_TYPE (PhotosFlickrItem, photos_flickr_item, PHOTOS_TYPE_BASE_ITEM);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosFlickrItem, photos_flickr_item, PHOTOS_TYPE_BASE_ITEM);
 
 
 typedef struct _PhotosFlickrItemSyncData PhotosFlickrItemSyncData;
@@ -125,9 +126,8 @@ photos_flickr_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancell
   identifier = photos_base_item_get_identifier (item);
   if (strlen (identifier) <= prefix_len || !g_str_has_prefix (identifier, flickr_prefix))
     {
-      /* FIXME: use proper #defines and enumerated types */
       g_set_error (error,
-                   g_quark_from_static_string ("gnome-photos-error"),
+                   PHOTOS_ERROR,
                    0,
                    "Invalid nao:identifier for Flickr item %s",
                    identifier);
@@ -139,9 +139,8 @@ photos_flickr_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancell
   resource_urn = photos_base_item_get_resource_urn (item);
   if (strlen (resource_urn) <= prefix_len || !g_str_has_prefix (resource_urn, resource_prefix))
     {
-      /* FIXME: use proper #defines and enumerated types */
       g_set_error (error,
-                   g_quark_from_static_string ("gnome-photos-error"),
+                   PHOTOS_ERROR,
                    0,
                    "Invalid nie:dataSource for Flickr item %s",
                    resource_urn);
@@ -154,9 +153,8 @@ photos_flickr_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancell
   source = grl_registry_lookup_source (registry, grilo_id);
   if (source == NULL)
     {
-      /* FIXME: use proper #defines and enumerated types */
       g_set_error (error,
-                   g_quark_from_static_string ("gnome-photos-error"),
+                   PHOTOS_ERROR,
                    0,
                    "Failed to find a GrlSource for %s",
                    grilo_id);
@@ -252,7 +250,10 @@ photos_flickr_item_download (PhotosBaseItem *item, GCancellable *cancellable, GE
                         NULL,
                         NULL,
                         error))
-        goto out;
+        {
+          g_file_delete (local_file, NULL, NULL);
+          goto out;
+        }
     }
 
   ret_val = local_path;
@@ -338,7 +339,7 @@ photos_flickr_item_init (PhotosFlickrItem *self)
 {
   PhotosFlickrItemPrivate *priv;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_FLICKR_ITEM, PhotosFlickrItemPrivate);
+  self->priv = photos_flickr_item_get_instance_private (self);
   priv = self->priv;
 
   priv->src_mngr = photos_source_manager_new ();
@@ -357,8 +358,6 @@ photos_flickr_item_class_init (PhotosFlickrItemClass *class)
   base_item_class->download = photos_flickr_item_download;
   base_item_class->get_source_name = photos_flickr_item_get_source_name;
   base_item_class->open = photos_flickr_item_open;
-
-  g_type_class_add_private (class, sizeof (PhotosFlickrItemPrivate));
 }
 
 
