@@ -59,7 +59,7 @@ struct _PhotosMainToolbarPrivate
 };
 
 
-G_DEFINE_TYPE (PhotosMainToolbar, photos_main_toolbar, GTK_TYPE_BOX);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosMainToolbar, photos_main_toolbar, GTK_TYPE_BOX);
 
 
 static void
@@ -160,7 +160,7 @@ photos_main_toolbar_coll_back_button_clicked (GtkButton *button, gpointer user_d
 
 
 static void
-photos_main_toolbar_active_changed (PhotosBaseManager *manager, GObject *object, gpointer user_data)
+photos_main_toolbar_col_active_changed (PhotosBaseManager *manager, GObject *object, gpointer user_data)
 {
   PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
   PhotosMainToolbarPrivate *priv = self->priv;
@@ -197,6 +197,22 @@ photos_main_toolbar_active_changed (PhotosBaseManager *manager, GObject *object,
 }
 
 
+static void
+photos_main_toolbar_item_active_changed (PhotosBaseManager *manager, GObject *object, gpointer user_data)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
+  PhotosMainToolbarPrivate *priv = self->priv;
+  PhotosWindowMode window_mode;
+
+  if (object == NULL)
+    return;
+
+  window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_main_toolbar_set_toolbar_title (self);
+}
+
+
 static void
 photos_main_toolbar_select_button_clicked (GtkButton *button, gpointer user_data)
 {
@@ -261,7 +277,7 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
 
   g_signal_connect_object (priv->col_mngr,
                            "active-changed",
-                           G_CALLBACK (photos_main_toolbar_active_changed),
+                           G_CALLBACK (photos_main_toolbar_col_active_changed),
                            self,
                            0);
 }
@@ -296,7 +312,10 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
     }
 
   if (priv->col_mngr != NULL)
-    g_signal_handlers_disconnect_by_func (priv->col_mngr, photos_main_toolbar_active_changed, self);
+    g_signal_handlers_disconnect_by_func (priv->col_mngr, photos_main_toolbar_col_active_changed, self);
+
+  if (priv->item_mngr != NULL)
+    g_signal_handlers_disconnect_by_func (priv->item_mngr, photos_main_toolbar_item_active_changed, self);
 
   if (priv->sel_cntrlr != NULL)
     g_signal_handlers_disconnect_by_func (priv->sel_cntrlr, photos_main_toolbar_set_toolbar_title, self);
@@ -309,6 +328,7 @@ photos_main_toolbar_clear_toolbar (PhotosMainToolbar *self)
   PhotosMainToolbarPrivate *priv = self->priv;
 
   photos_main_toolbar_clear_state_data (self);
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), FALSE);
   photos_header_bar_clear (PHOTOS_HEADER_BAR (priv->toolbar));
   g_simple_action_set_enabled (priv->gear_menu, FALSE);
 }
@@ -364,11 +384,12 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *object;
 
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_active_changed (priv->col_mngr, object, self);
+  photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
 }
 
 
@@ -396,11 +417,12 @@ photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *object;
 
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_active_changed (priv->col_mngr, object, self);
+  photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
 }
 
 
@@ -410,11 +432,12 @@ photos_main_toolbar_populate_for_overview (PhotosMainToolbar *self)
   PhotosMainToolbarPrivate *priv = self->priv;
   GObject *object;
 
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_NORMAL);
   photos_main_toolbar_add_selection_button (self);
 
   object = photos_base_manager_get_active_object (priv->col_mngr);
-  photos_main_toolbar_active_changed (priv->col_mngr, object, self);
+  photos_main_toolbar_col_active_changed (priv->col_mngr, object, self);
 }
 
 
@@ -429,6 +452,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   gboolean remote_display_available;
   GAction *remote_display_action;
 
+  gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (priv->toolbar), TRUE);
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
 
   back_button = photos_main_toolbar_add_back_button (self);
@@ -449,6 +473,8 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   remote_display_action = g_action_map_lookup_action (G_ACTION_MAP (app), "remote-display-current");
   remote_display_available = photos_dlna_renderers_manager_is_available ();
   g_simple_action_set_enabled (G_SIMPLE_ACTION (remote_display_action), remote_display_available);
+
+  g_signal_connect (priv->item_mngr, "active-changed", G_CALLBACK (photos_main_toolbar_item_active_changed), self);
 }
 
 
@@ -462,7 +488,7 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_SELECTION);
 
   selection_button = gd_header_simple_button_new ();
-  gd_header_button_set_label (GD_HEADER_BUTTON (selection_button), _("Done"));
+  gd_header_button_set_label (GD_HEADER_BUTTON (selection_button), _("Cancel"));
   gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), selection_button);
   context = gtk_widget_get_style_context (selection_button);
   gtk_style_context_add_class (context, "suggested-action");
@@ -563,7 +589,7 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
   GtkApplication *app;
   GtkBuilder *builder;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_MAIN_TOOLBAR, PhotosMainToolbarPrivate);
+  self->priv = photos_main_toolbar_get_instance_private (self);
   priv = self->priv;
 
   gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
@@ -641,8 +667,6 @@ photos_main_toolbar_class_init (PhotosMainToolbarClass *class)
   GObjectClass *object_class = G_OBJECT_CLASS (class);
 
   object_class->dispose = photos_main_toolbar_dispose;
-
-  g_type_class_add_private (class, sizeof (PhotosMainToolbarPrivate));
 }
 
 
