@@ -30,17 +30,28 @@
 #include <glib/gi18n.h>
 
 #include "gegl-gtk-view.h"
+#include "photos-mode-controller.h"
+#include "photos-preview-nav-buttons.h"
 #include "photos-preview-view.h"
 
 
 struct _PhotosPreviewViewPrivate
 {
   GeglNode *node;
+  GtkWidget *overlay;
   GtkWidget *view;
+  PhotosModeController *mode_cntrlr;
+  PhotosPreviewNavButtons *nav_buttons;
+};
+
+enum
+{
+  PROP_0,
+  PROP_OVERLAY
 };
 
 
-G_DEFINE_TYPE (PhotosPreviewView, photos_preview_view, GTK_TYPE_SCROLLED_WINDOW);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosPreviewView, photos_preview_view, GTK_TYPE_SCROLLED_WINDOW);
 
 
 static void
@@ -109,6 +120,14 @@ photos_preview_view_scale_and_align_image (PhotosPreviewView *self)
 }
 
 
+static void
+photos_preview_view_window_mode_changed (PhotosPreviewView *self, PhotosWindowMode mode, PhotosWindowMode old_mode)
+{
+  if (mode != PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_preview_nav_buttons_hide (self->priv->nav_buttons);
+}
+
+
 static void
 photos_preview_view_dispose (GObject *object)
 {
@@ -116,6 +135,7 @@ photos_preview_view_dispose (GObject *object)
   PhotosPreviewViewPrivate *priv = self->priv;
 
   g_clear_object (&priv->node);
+  g_clear_object (&priv->mode_cntrlr);
 
   G_OBJECT_CLASS (photos_preview_view_parent_class)->dispose (object);
 }
@@ -134,19 +154,46 @@ photos_preview_view_constructed (GObject *object)
    */
   gtk_container_add (GTK_CONTAINER (self), priv->view);
 
+  priv->nav_buttons = photos_preview_nav_buttons_new (self, GTK_OVERLAY (priv->overlay));
+
   gtk_widget_show_all (GTK_WIDGET (self));
 }
 
 
+static void
+photos_preview_view_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+{
+  PhotosPreviewView *self = PHOTOS_PREVIEW_VIEW (object);
+  PhotosPreviewViewPrivate *priv = self->priv;
+
+  switch (prop_id)
+    {
+    case PROP_OVERLAY:
+      priv->overlay = GTK_WIDGET (g_value_dup_object (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+
 static void
 photos_preview_view_init (PhotosPreviewView *self)
 {
   PhotosPreviewViewPrivate *priv;
   GtkStyleContext *context;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_PREVIEW_VIEW, PhotosPreviewViewPrivate);
+  self->priv = photos_preview_view_get_instance_private (self);
   priv = self->priv;
 
+  priv->mode_cntrlr = photos_mode_controller_new ();
+  g_signal_connect_swapped (priv->mode_cntrlr,
+                            "window-mode-changed",
+                            G_CALLBACK (photos_preview_view_window_mode_changed),
+                            self);
+
   gtk_widget_set_hexpand (GTK_WIDGET (self), TRUE);
   gtk_widget_set_vexpand (GTK_WIDGET (self), TRUE);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self), GTK_SHADOW_IN);
@@ -154,6 +201,7 @@ photos_preview_view_init (PhotosPreviewView *self)
   gtk_style_context_add_class (context, "documents-scrolledwin");
 
   priv->view = GTK_WIDGET (gegl_gtk_view_new ());
+  gtk_widget_add_events (priv->view, GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
   context = gtk_widget_get_style_context (priv->view);
   gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
   gtk_style_context_add_class (context, "content-view");
@@ -175,15 +223,29 @@ photos_preview_view_class_init (PhotosPreviewViewClass *class)
 
   object_class->constructed = photos_preview_view_constructed;
   object_class->dispose = photos_preview_view_dispose;
+  object_class->set_property = photos_preview_view_set_property;
 
-  g_type_class_add_private (class, sizeof (PhotosPreviewViewPrivate));
+  g_object_class_install_property (object_class,
+                                   PROP_OVERLAY,
+                                   g_param_spec_object ("overlay",
+                                                        "GtkOverlay object",
+                                                        "The stack overlay widget",
+                                                        GTK_TYPE_OVERLAY,
+                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 }
 
 
 GtkWidget *
-photos_preview_view_new (void)
+photos_preview_view_new (GtkOverlay *overlay)
 {
-  return g_object_new (PHOTOS_TYPE_PREVIEW_VIEW, NULL);
+  return g_object_new (PHOTOS_TYPE_PREVIEW_VIEW, "overlay", overlay, NULL);
+}
+
+
+void
+photos_preview_view_set_model (PhotosPreviewView *self, GtkTreeModel *model, GtkTreePath *current_path)
+{
+  photos_preview_nav_buttons_set_model (self->priv->nav_buttons, model, current_path);
 }
 
 
@@ -193,15 +255,19 @@ photos_preview_view_set_node (PhotosPreviewView *self, GeglNode *node)
   PhotosPreviewViewPrivate *priv = self->priv;
 
   if (priv->node == node)
-    return;
+    goto out;
 
   g_clear_object (&priv->node);
   if (node == NULL)
-    return;
+    goto out;
 
   priv->node = g_object_ref (node);
   photos_preview_view_scale_and_align_image (self);
 
   /* Steals the reference to the GeglNode. */
   gegl_gtk_view_set_node (GEGL_GTK_VIEW (priv->view), g_object_ref (priv->node));
+  gtk_widget_queue_draw (priv->view);
+
+ out:
+  photos_preview_nav_buttons_show (priv->nav_buttons);
 }
