@@ -104,7 +104,7 @@ photos_query_builder_filter (gint flags)
 static gchar *
 photos_query_builder_optional (void)
 {
-  return g_strdup ("OPTIONAL { ?urn nco:creator ?creater . } "
+  return g_strdup ("OPTIONAL { ?urn nco:creator ?creator . } "
                    "OPTIONAL { ?urn nco:publisher ?publisher . }");
 }
 
@@ -198,7 +198,15 @@ photos_query_builder_query (gboolean global, gint flags)
                         "nie:dataSource(?urn) "
                         "( EXISTS { ?urn nao:hasTag nao:predefined-tag-favorite } ) "
                         "( EXISTS { ?urn nco:contributor ?contributor FILTER ( ?contributor != ?creator ) } ) "
-                        "tracker:coalesce(nfo:fileCreated (?urn), nie:contentCreated (?urn))",
+                        "tracker:coalesce(nfo:fileCreated (?urn), nie:contentCreated (?urn))"
+                        "nfo:width (?urn)"
+                        "nfo:height (?urn)"
+                        "nfo:equipment (?urn)"
+                        "nmm:exposureTime (?urn)"
+                        "nmm:fnumber (?urn)"
+                        "nmm:focalLength (?urn)"
+                        "nmm:isoSpeed (?urn)"
+                        "nmm:flash (?urn)",
                         where_sparql,
                         tail_sparql,
                         NULL);
@@ -274,6 +282,18 @@ photos_query_builder_delete_resource_query (const gchar *resource)
 }
 
 
+PhotosQuery *
+photos_query_builder_equipment_query (GQuark equipment)
+{
+  const gchar *resource;
+  gchar *sparql;
+
+  resource = g_quark_to_string (equipment);
+  sparql = g_strdup_printf ("SELECT nfo:manufacturer (<%s>) nfo:model (<%s>) WHERE {}", resource, resource);
+  return photos_query_new (sparql);
+}
+
+
 PhotosQuery *
 photos_query_builder_fetch_collections_query (const gchar *resource)
 {
