@@ -49,7 +49,7 @@ enum
 };
 
 
-G_DEFINE_TYPE (PhotosSingleItemJob, photos_single_item_job, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosSingleItemJob, photos_single_item_job, G_TYPE_OBJECT);
 
 
 static void
@@ -166,7 +166,7 @@ photos_single_item_job_init (PhotosSingleItemJob *self)
 {
   PhotosSingleItemJobPrivate *priv = self->priv;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_SINGLE_ITEM_JOB, PhotosSingleItemJobPrivate);
+  self->priv = photos_single_item_job_get_instance_private (self);
   priv = self->priv;
 
   priv->queue = photos_tracker_queue_new ();
@@ -189,8 +189,6 @@ photos_single_item_job_class_init (PhotosSingleItemJobClass *class)
                                                         "An unique ID associated with this item",
                                                         NULL,
                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
-
-  g_type_class_add_private (class, sizeof (PhotosSingleItemJobPrivate));
 }
 
 
