@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -70,6 +70,12 @@ GType                     photos_source_manager_get_type           (void) G_GNUC
 
 PhotosBaseManager        *photos_source_manager_new                (void);
 
+GList                    *photos_source_manager_get_for_provider_type (PhotosSourceManager *self,
+                                                                       const gchar *provider_type);
+
+gboolean                  photos_source_manager_has_provider_type     (PhotosSourceManager *self,
+                                                                       const gchar *provider_type);
+
 G_END_DECLS
 
 #endif /* PHOTOS_SOURCE_MANAGER_H */
