@@ -33,9 +33,9 @@ struct _PhotosTrackerFavoritesControllerPrivate
 };
 
 
-G_DEFINE_TYPE (PhotosTrackerFavoritesController,
-               photos_tracker_favorites_controller,
-               PHOTOS_TYPE_TRACKER_CONTROLLER);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosTrackerFavoritesController,
+                            photos_tracker_favorites_controller,
+                            PHOTOS_TYPE_TRACKER_CONTROLLER);
 
 
 static PhotosOffsetController *
@@ -98,9 +98,7 @@ photos_tracker_favorites_controller_init (PhotosTrackerFavoritesController *self
 {
   PhotosTrackerFavoritesControllerPrivate *priv;
 
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                            PHOTOS_TYPE_TRACKER_FAVORITES_CONTROLLER,
-                                            PhotosTrackerFavoritesControllerPrivate);
+  self->priv = photos_tracker_favorites_controller_get_instance_private (self);
   priv = self->priv;
 
   priv->col_mngr = photos_collection_manager_new ();
@@ -117,8 +115,6 @@ photos_tracker_favorites_controller_class_init (PhotosTrackerFavoritesController
   object_class->dispose = photos_tracker_favorites_controller_dispose;
   tracker_controller_class->get_offset_controller = photos_tracker_favorites_controller_get_offset_controller;
   tracker_controller_class->get_query = photos_tracker_favorites_controller_get_query;
-
-  g_type_class_add_private (class, sizeof (PhotosTrackerFavoritesControllerPrivate));
 }
 
 
