@@ -358,6 +358,13 @@ photos_utils_embed_image_in_frame (GdkPixbuf *source_image,
 }
 
 
+GQuark
+photos_utils_error_quark (void)
+{
+  return g_quark_from_static_string ("gnome-photos-error-quark");
+}
+
+
 static gchar *
 photos_utils_filename_get_extension_offset (const gchar *filename)
 {
@@ -404,6 +411,20 @@ photos_utils_filename_strip_extension (const gchar *filename_with_extension)
 }
 
 
+GQuark
+photos_utils_flash_off_quark (void)
+{
+  return g_quark_from_static_string ("http://www.tracker-project.org/temp/nmm#flash-off");
+}
+
+
+GQuark
+photos_utils_flash_on_quark (void)
+{
+  return g_quark_from_static_string ("http://www.tracker-project.org/temp/nmm#flash-on");
+}
+
+
 gint
 photos_utils_get_icon_size (void)
 {
