@@ -44,6 +44,7 @@ struct _PhotosViewContainerPrivate
 {
   GdMainView *view;
   GtkListStore *model;
+  GtkTreePath *current_path;
   GtkWidget *load_more;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
@@ -61,7 +62,7 @@ enum
 };
 
 
-G_DEFINE_TYPE (PhotosViewContainer, photos_view_container, GTK_TYPE_GRID);
+G_DEFINE_TYPE_WITH_PRIVATE (PhotosViewContainer, photos_view_container, GTK_TYPE_GRID);
 
 
 static void
@@ -153,6 +154,7 @@ photos_view_container_item_activated (GdMainView *main_view,
   PhotosViewContainerPrivate *priv = self->priv;
   GObject *object;
 
+  priv->current_path = gtk_tree_path_copy (path);
   object = photos_base_manager_get_object_by_id (priv->item_mngr, id);
 
   if (!photos_base_item_is_collection (PHOTOS_BASE_ITEM (object)) &&
@@ -352,6 +354,18 @@ photos_view_container_dispose (GObject *object)
 }
 
 
+static void
+photos_view_container_finalize (GObject *object)
+{
+  PhotosViewContainer *self = PHOTOS_VIEW_CONTAINER (object);
+  PhotosViewContainerPrivate *priv = self->priv;
+
+  g_clear_pointer (&priv->current_path, (GDestroyNotify) gtk_tree_path_free);
+
+  G_OBJECT_CLASS (photos_view_container_parent_class)->finalize (object);
+}
+
+
 static void
 photos_view_container_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
@@ -373,9 +387,7 @@ photos_view_container_set_property (GObject *object, guint prop_id, const GValue
 static void
 photos_view_container_init (PhotosViewContainer *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                            PHOTOS_TYPE_VIEW_CONTAINER,
-                                            PhotosViewContainerPrivate);
+  self->priv = photos_view_container_get_instance_private (self);
 }
 
 
@@ -386,6 +398,7 @@ photos_view_container_class_init (PhotosViewContainerClass *class)
 
   object_class->constructed = photos_view_container_constructed;
   object_class->dispose = photos_view_container_dispose;
+  object_class->finalize = photos_view_container_finalize;
   object_class->set_property = photos_view_container_set_property;
 
   g_object_class_install_property (object_class,
@@ -396,8 +409,6 @@ photos_view_container_class_init (PhotosViewContainerClass *class)
                                                       PHOTOS_TYPE_WINDOW_MODE,
                                                       PHOTOS_WINDOW_MODE_NONE,
                                                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
-
-  g_type_class_add_private (class, sizeof (PhotosViewContainerPrivate));
 }
 
 
@@ -406,3 +417,17 @@ photos_view_container_new (PhotosWindowMode mode)
 {
   return g_object_new (PHOTOS_TYPE_VIEW_CONTAINER, "mode", mode, NULL);
 }
+
+
+GtkTreePath *
+photos_view_container_get_current_path (PhotosViewContainer *self)
+{
+  return self->priv->current_path;
+}
+
+
+GtkListStore *
+photos_view_container_get_model (PhotosViewContainer *self)
+{
+  return self->priv->model;
+}
